#!/usr/bin/python
# 
# Copyright (C) 2009, Intel Corporation.
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms and conditions of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.  
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307 USA.
#
# Authors:
#              Wei, Zhang <wei.z.zhang@intel.com> 
#              Hao, Chen <hao.h.chen@intel.com>
# condition:
# 1. ldconfig can search the target so
# 2. so libraries are in their meta-packages
# steps:
# 1. ldconfig to locate so according to exact soname
# 2. rpm/dpkg manager to extract so related package version
# 3.1. no package. WARNING to install related package
# 3.2.1. one package found. PASS for >=[required version]
# 3.2.2. one package found. WARNING for <[required version]
# 3.3. more than one package. WARNING to check these packages manually from output.

import os
import sys
import re
from distutils.sysconfig import get_python_lib

sys.path.append('../../tetj/')
from tetj import *

sys.path.append('../utils/')
from shell_exec import *

M2CT_LIBS_FILE = 'data/M2CT_LIBS'

# Parse package version from package name
def __parse_pkg_version(pkg_ver_str):

    pkg_ver_str = "\n" + pkg_ver_str
    res = re.findall(r'[-:\s]([\d\.-]+\d)', pkg_ver_str)

    if len(res) != 0:
        return res[0]
    else:
        return None

# Get package version
def get_pkg_info(soname, is_deb_os):

    if is_deb_os == False:
        # RPM package
        # -----------
	fsonames = shell_exec1("/sbin/ldconfig -p | grep -F '\t%s ' | awk -F\"=> \" '{print $2}'" % soname).split('\n')
	nvs = []

	for fsoname in fsonames:
	    if fsoname != "":
                nv = shell_exec1("rpm -qf --qf '%%{name}\\t%%{version}\\n' %s" % fsoname).split('\t')
	        nvs.append([nv[0], __parse_pkg_version(nv[1]), fsoname])

	return nvs

    else:
        # Debian package
        # --------------
	fsonames = shell_exec1("/sbin/ldconfig -p | grep -F '\t%s ' | awk -F\"=> \" '{print $2}'" % soname).split('\n')
	nvs = []
	
	for fsoname in fsonames:
	    if fsoname != "":
                pkgname = shell_exec1("dpkg --search '%s' | awk -F: '{print $1}'" % fsoname)
		if pkgname != "":
		    nv = shell_exec1("dpkg-query -W -f='${Package}\\t${Version}\\n' '%s'" % pkgname).split('\t')
		    nvs.append([nv[0], __parse_pkg_version(nv[1]), fsoname])

	return nvs


# compare version using array-compare 
def compare_pkg_ver(cur_ver, req_ver, is_above_req):
    # not allowed None
    if cur_ver == None or req_ver == None:
        return False

    # deal with '-'
    cur_ver = cur_ver.replace('-', '.')
    req_ver = req_ver.replace('-', '.')

    # split 
    cur_vers = cur_ver.split('.')
    req_vers = req_ver.split('.')
    
    # Convert list members to integer type
    cur_vers_int = [ int(num) for num in cur_vers ]
    req_vers_int = [ int(num) for num in req_vers ]

    # Compare 2 lists
    if is_above_req:
	return cur_vers_int >= req_vers_int
    else:
	return cur_vers_int == req_vers_int

# Test body
def libpkgchk(journal):

    import csv
    try:
	# judge os
        is_deb_os = False
        if os.path.exists("/etc/debian_version"):
            is_deb_os = True

        # read required
	print 'Read libs requirement file: ' + M2CT_LIBS_FILE
	reader = csv.reader(open(M2CT_LIBS_FILE))

	# check every required
	for name, soname, moblin_req_ver, upper_ver in reader:
            name, soname, moblin_req_ver, upper_ver = [ field.strip() for field in (name, soname, moblin_req_ver, upper_ver) ]

	    journal.testcase_start("Library-Version-Checking-" + soname)
	    journal.purpose_start()
	    ret = TETJ_FIP

	    info = ('Checked requirement - library <%s>\n' % soname)
 
            try:

		# Get package version
		info += "[1] get package for library <%s>\n" %soname
                pkg_infos = get_pkg_info(soname, is_deb_os)

		# no package for specified soname found
		if len(pkg_infos) == 0:
		    info += '[WARNING] No package found for library <%s> . Try to install <%s> related package\n' %(soname, name)
		else:
		    info += "[2] compare version for package(s)\n"

		# only one package for specified soname found
		if len(pkg_infos) == 1:
		    pkg_info = pkg_infos[0]
                    if compare_pkg_ver(pkg_info[1], moblin_req_ver, (upper_ver == '+')):
		        info += '[pass]  <%s> in package <%s> - current version: <%s>, moblin required version: <%s>%s\n' \
                                    % (pkg_info[2], pkg_info[0], pkg_info[1], moblin_req_ver, upper_ver)
			ret = TETJ_PASS
		    else:
		        info += '[fail]  <%s> in package <%s> - current version: <%s>, moblin required version: <%s>%s\n' \
                                    % (pkg_info[2], pkg_info[0], pkg_info[1], moblin_req_ver, upper_ver)
			ret = TETJ_FIP

		    if TETJ_PASS == ret:
		        info += "[PASS] package <%s> found for library <%s> with acceptable version <%s>\n" %(pkg_info[0],soname,pkg_info[1])
		    if TETJ_FIP == ret:
		        info += "[WARNING] package <%s> found for library <%s> with lower version than required\n" %(pkg_info[0],soname)

		# more than one package for specified soname found
		if len(pkg_infos) > 1:
		    ret = TETJ_FIP
		    # Compare package version to M2CT_LIBS requirement
                    for pkg_info in pkg_infos:
                        if compare_pkg_ver(pkg_info[1], moblin_req_ver, (upper_ver == '+')):
		            info += '[pass]  <%s> in package <%s> - current version: <%s>, moblin required version: <%s>%s\n' \
                                    % (pkg_info[2], pkg_info[0], pkg_info[1], moblin_req_ver, upper_ver)
		        else:
		            info += '[fail]  <%s> in package <%s> - current version: <%s>, moblin required version: <%s>%s\n' \
                                    % (pkg_info[2], pkg_info[0], pkg_info[1], moblin_req_ver, upper_ver)
		    info += "[WARNING] more than one package found for library, please check manually from the output above\n"

            except:
                info += "Exception on testing"	

	    # Print final result
	    print info

	    # finish tet log
	    journal.testcase_info(0, 0, 0, info)	  
	    journal.result(ret)
	    journal.purpose_end()
	    journal.testcase_end()

    except (OSError, IOError), (errno, strerror):
        print "error(%s): %s" % (errno, strerror)

    

if __name__ == '__main__':

    # Check command line parameters, like "-j journal_file"
    if (len(sys.argv) == 3) and (sys.argv[1] == '-j'):
        journal_file = sys.argv[2]
    else:
        sys.stderr.write('Usage: %s -j journal_file\n' % sys.argv[0])
        sys.exit(1)
    
    try:
	journal = Journal(journal_file, 'Moblin Compliance Test - library version check')
    except  IOError:
        sys.stderr.write('Cannot create journal file : %s\n' % journal_file)
        sys.exit(1)

    # libpkgchk
    libpkgchk (journal)

    journal.close()
